"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireWildcard(require("path"));

var _fs = require("fs");

var _core = require("@babel/core");

var _graphqlTag = _interopRequireDefault(require("graphql-tag"));

var _multiOp = require("./multi-op");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var resolve;

var _default = function _default(_ref) {
  var t = _ref.types;
  return {
    manipulateOptions(_ref2) {
      var resolveModuleSource = _ref2.resolveModuleSource;

      if (!resolve) {
        resolve = resolveModuleSource || function (src, file) {
          return _path.default.resolve((0, _path.dirname)(file), src);
        };
      }
    },

    visitor: {
      ImportDeclaration: {
        exit(curPath, _ref3) {
          var babelPath = _ref3.file.opts.filename,
              nodePath = _ref3.opts.nodePath;
          var importPath = curPath.node.source.value;

          if (importPath.endsWith('.graphql') || importPath.endsWith('.gql')) {
            var fallbackPaths = nodePath ? nodePath.split(_path.delimiter) : process.env.NODE_PATH;
            var doc = processDoc(createDoc(importPath, babelPath, fallbackPaths));
            var replacements = buildReplacements((0, _multiOp.createDocPerOp)(doc), curPath.node.specifiers);
            replacements.length > 1 ? curPath.replaceWithMultiple(replacements) : curPath.replaceWith(replacements[0]);
          }

          function buildReplacements(docs, specifiers) {
            return specifiers.map(function (_ref4) {
              var type = _ref4.type,
                  imported = _ref4.imported,
                  local = _ref4.local;

              switch (type) {
                case 'ImportDefaultSpecifier':
                  return buildVariableAST(docs.default, local.name);

                case 'ImportSpecifier':
                  return buildVariableAST(docs[imported.name] || docs.default, local.name);

                case 'ImportNamespaceSpecifier':
                  return buildVariableAST(docs, local.name);

                default:
                  throw new Error(`Unexpected import specifier type: ${type}`);
              }
            });
          }

          function buildVariableAST(graphqlAST, importName) {
            var _transformSync = (0, _core.transformSync)(`var ${importName} = ${JSON.stringify(graphqlAST)}`),
                ast = _transformSync.ast;

            return ast.program.body[0];
          }
        }

      }
    }
  };
};

exports.default = _default;

function createDoc(importPath, babelPath, paths) {
  var absPath = resolve(importPath, babelPath);
  if (!(0, _fs.existsSync)(absPath)) absPath = require.resolve(importPath, {
    paths
  });
  var source = (0, _fs.readFileSync)(absPath).toString();
  var ast = null;
  var fragmentDefs = [];
  return {
    processFragments() {
      processImports(getImportStatements(source), absPath);

      function getImportStatements(src) {
        return src.split(/(\r\n|\r|\n)+/).filter(function (line) {
          return line.startsWith('#import');
        });
      }

      function processImports(imports, relFile) {
        imports.forEach(function (statement) {
          var fragmentPath = statement.split(/[\s\n]+/g)[1].slice(1, -1);
          var absFragmentPath = resolve(fragmentPath, relFile);
          var fragmentSource = (0, _fs.readFileSync)(absFragmentPath.replace(/'/g, '')).toString();
          var subFragments = getImportStatements(fragmentSource);
          if (subFragments.length > 0) processImports(subFragments, absFragmentPath); // prettier-ignore

          fragmentDefs = _toConsumableArray(_graphqlTag.default`${fragmentSource}`.definitions).concat(_toConsumableArray(fragmentDefs));
        });
      }
    },

    parse() {
      // prettier-ignore
      var parsedAST = _graphqlTag.default`${source}`;
      parsedAST.definitions = _toConsumableArray(parsedAST.definitions).concat(_toConsumableArray(fragmentDefs));
      ast = parsedAST;
    },

    dedupeFragments() {
      var seenNames = {};
      ast.definitions = ast.definitions.filter(function (def) {
        if (def.kind !== 'FragmentDefinition') return true;
        return seenNames[def.name.value] ? false : seenNames[def.name.value] = true;
      });
    },

    makeSourceEnumerable() {
      var newAST = JSON.parse(JSON.stringify(ast));
      newAST.loc.source = ast.loc.source;
      ast = newAST;
    },

    get ast() {
      return ast;
    }

  };
}

function processDoc(doc) {
  doc.processFragments();
  doc.parse();
  doc.dedupeFragments();
  doc.makeSourceEnumerable();
  return doc.ast;
}